module Main where

import Control.Monad
import Data.List
import Data.Monoid
import Distribution.Text
import Distribution.Version
import Distribution.Package
import Options.Applicative
import System.Directory
import System.Exit
import qualified Data.ByteString as BS
import qualified Control.Exception as Exception
import System.Environment (getEnvironment)

-- import Paths_ghc_ppa_tools (getDataFileName)

main :: IO ()
main = execParser cmds >>= run
  where
    cmds :: ParserInfo Cmd
    cmds = info (sample <**> helper) idm

data Cmd
  = BashInit !Int
  | Collection String
  | NOP
  deriving (Eq, Show)

bashInit :: Parser Cmd
bashInit = BashInit <$> argument auto (metavar "VERSION")

collection :: Parser Cmd
collection = Collection <$> argument str (metavar "COLLECTION")

sample :: Parser Cmd
sample = subparser $ mconcat
         [ command "bash-init"
           (info bashInit (progDesc "print filename of helper functions"))
         , command "collection"
           (info collection (progDesc "dump collection to stdout (use 'list' to show list of available collections)"))
         , command "nop"
           (info (pure NOP) (progDesc "no-op"))
         ]

-- hack
getDataFileName :: FilePath -> IO FilePath
getDataFileName name = do
    dir <- getDataDir
    return (dir ++ "/" ++ name)
  where
    getEnv k = do
        env <- getEnvironment
        pure (lookup k env)
    getDataDir = maybe "/opt/ghc-ppa-tools/share" id <$> getEnv "ghc_ppa_tools_datadir"

getColls :: IO (FilePath, [((String,Version),FilePath)])
getColls = do
    ltsfolder <- getDataFileName "collection/lts"
    ents <- listDirectory ltsfolder

    return (ltsfolder, sort [ (cid,fn) | fn <- ents, Just cid <- [go fn] ])
  where
    go :: FilePath -> Maybe (String,Version)
    go fp | Just x <- stripSuffix ".freeze" fp = case simpleParse (map (\c -> if c=='_' then '.' else c) x) of
                                                   Just v -> Just ("lts",v)
                                                   Nothing -> Nothing

    stripSuffix y = fmap reverse . stripPrefix (reverse y) . reverse

normV :: Version -> [Version] -> Version
normV v vs
  | v `elem` vs = v
  | v':_ <- vs' = v'
  | otherwise = v
  where
    isPfxV (Version v1 _) (Version v2 _) = v1 `isPrefixOf` v2
    vs' = filter (isPfxV v) $ reverse (sort vs)

run :: Cmd -> IO ()
run (BashInit v) = case v of
  1 -> putStrLn =<< getDataFileName "bash-init/v1.sh"
  _ -> die "unsupported version"

run (Collection "list") = do
    (_, colls) <- getColls
    forM_ colls $ \((n,v),_) -> do
        putStrLn (n ++ "-" ++ display v)
run (Collection cid) = do
    case simpleParse cid of
      Nothing -> die "invalid collection-id"
      Just (PackageIdentifier n0 v) -> do
          let n = unPackageName n0

          (basefolder, colls) <- getColls
          let vs = [ v' | ((n',v'),_) <- colls, n' == n ]
          case lookup (n,normV v vs) colls of
            Nothing -> die "collection-id not found"
            Just fn -> BS.putStrLn =<< BS.readFile (basefolder ++ "/" ++ fn)

    pure ()

run NOP = pure ()
